Package: jmv
Type: Package
Title: The 'jamovi' Analyses
Version: 2.5.6
Date: 2024-06-19
Authors@R: c(person("Ravi", "Selker", role=c("aut", "cph")),
    person("Jonathon", "Love", role=c("aut", "cre", "cph"), email="jon@thon.cc"),
    person("Damian", "Dropmann", role=c("aut", "cph")),
    person("Victor", "Moreno", role=c("ctb", "cph")),
    person("Maurizio", "Agosti", role=c("ctb", "cph")))
Maintainer: Jonathon Love <jon@thon.cc>
Description: A suite of common statistical methods such as descriptives,
    t-tests, ANOVAs, regression, correlation matrices, proportion tests,
    contingency tables, and factor analysis. This package is also useable from
    the 'jamovi' statistical spreadsheet (see <https://www.jamovi.org> for more
    information).
BugReports: https://github.com/jamovi/jmv/issues
License: GPL (>= 2)
Depends: R (>= 3.2)
Imports: jmvcore (>= 2.4.2), R6, car (>= 3.0.0), multcomp, ggplot2 (>=
        2.2.1), PMCMR, emmeans (>= 1.4.2), vcd, vcdExtra, GGally,
        BayesFactor, psych (>= 1.7.5), GPArotation, afex (>= 0.28-0),
        mvnormtest, lavaan, ggridges, ROCR, nnet, MASS, ggrepel, dplyr,
        magrittr
Suggests: exact2x2, testthat (>= 3.1.5), semPlot, carData, knitr,
        rmarkdown
Encoding: UTF-8
RoxygenNote: 6.1.1
NeedsCompilation: no
Packaged: 2024-06-19 09:36:16 UTC; c3113592
Author: Ravi Selker [aut, cph],
  Jonathon Love [aut, cre, cph],
  Damian Dropmann [aut, cph],
  Victor Moreno [ctb, cph],
  Maurizio Agosti [ctb, cph]
Repository: CRAN
Date/Publication: 2024-06-20 10:20:14 UTC
Built: R 4.4.1; ; 2024-06-21 19:47:48 UTC; unix
