Package: jmv
Type: Package
Title: The 'jamovi' Analyses
Version: 0.8.6.2
Date: 2018-04-06
Author: Ravi Selker, Jonathon Love, Damian Dropmann
Maintainer: Jonathon Love <jon@thon.cc>
Description: A suite of common statistical methods such as descriptives,
    t-tests, ANOVAs, regression, correlation matrices, proportion tests,
    contingency tables, and factor analysis. This package is also useable from
    the 'jamovi' statistical spreadsheet (see <https://www.jamovi.org> for more
    information).
License: GPL (>= 2)
Depends: R (>= 3.2)
Imports: jmvcore (>= 0.8.5), R6, car (>= 3.0.0), multcomp, ggplot2 (>=
        2.2.1), PMCMR, emmeans (>= 1.1.3), vcd, vcdExtra, GGally,
        BayesFactor, psych (>= 1.7.5), GPArotation, afex (>= 0.20.2),
        mvnormtest, lavaan, ggridges, ROCR, nnet, MASS
Suggests: exact2x2, testthat, semPlot, carData
Encoding: UTF-8
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2018-04-06 01:54:28 UTC; jonathon
Repository: CRAN
Date/Publication: 2018-04-06 04:06:25 UTC
Built: R 3.4.1; ; 2018-04-10 06:15:46 UTC; windows
