# Generated by roxygen2: do not edit by hand

export(ANOVA)
export(ancova)
export(anovaNP)
export(anovaOneW)
export(anovaRM)
export(anovaRMNP)
export(cfa)
export(contTables)
export(contTablesPaired)
export(corrMatrix)
export(corrPart)
export(descriptives)
export(efa)
export(linReg)
export(logLinear)
export(logRegBin)
export(logRegMulti)
export(logRegOrd)
export(mancova)
export(pca)
export(propTest2)
export(propTestN)
export(reliability)
export(ttestIS)
export(ttestOneS)
export(ttestPS)
import(ggplot2)
import(jmvcore)
importFrom(jmvcore,.)
importFrom(jmvcore,matchSet)
importFrom(magrittr,"%>%")
importFrom(stats,aggregate.data.frame)
importFrom(stats,complete.cases)
importFrom(stats,integrate)
importFrom(stats,pnorm)
